#!/usr/bin/sh
# set -x # for debugg purposes only
#VERSION="@(#) A.10.10		 $Revision: 1.10 $ $Date: 97/10/02 12:27:02 $"
# **********************************************************************
# **** Startup, Shutdown and Monitor Scripts for DB2 (Template) ***
# **********************************************************************
# ******* Note: This file MUST be edited before it can be used. ********
# **********************************************************************
#
# The following variables need to be set:		   
#
#
# MONITOR_INTERVAL: 	How often (in seconds) to monitor DB2's state.
#
#
# TIME_OUT:		The amount of time in seconds you want to wait for the
#			DB2 abort to complete before resorting to killing
#			the DB2 processes.  The TIME_OUT variable is to
#			protect against the worst case of a hung database
#			that would prevent the halt script from completing
#			and the standby node from starting up the data base.
#			The value of TIME_OUT must be less than the time out
#			variable set in the package configuration file.
#
#DB_TYPE:		The type of DB2 database being used.  The only valid options
#			are "SINGLE_PARTITION" or "MULTIPLE_PARTITION".
#			It is IMPORTANT that the correct DBtype is selected.
#			If the incorrect type is selected, the DB may not start.
# Examples:
# MONITOR_INTERVAL=5
# TIME_OUT=25
#DB_TYPE="SINGLE_PARTITION"

MONITOR_INTERVAL=5
TIME_OUT=25
#
# Only valid values for DB_TYPE are "SINGLE_PARTITION" or "MULTIPLE_PARTITION"
DB_TYPE="SINGLE_PARTITION"
#DB_TYPE="MULTIPLE_PARTITION"

#
# This version of the DB2.sh script no longer monitors individual
# DB2 processes.  An IBM DB2 utility, db2gcf, is now used to monitor
# the health of DB2.  The variable MONITOR_PROCESSES is now used
# as a fail-safe list of processes to ensure are not running at the
# time of shutdown.  If these processes are still running, they will
# be terminated with a "kill -9".
#
#
HOST=`hostname`
DATE=`date`
PATH=/sbin:/usr/sbin:/usr/bin
ACTION=${1}
DB2INSTANCE=${2}
DB2NODENUM=${3}
PACKAGE_NAME="${DB2INSTANCE}_${DB2NODENUM}" 


userhome=~${DB2INSTANCE?}
eval userhome=$userhome
INSTHOME=${userhome}
       
###############################################################################
# Function: DB2_run_cmds
#
# This function is used to start the DB2 instance.
###############################################################################

function DB2_run_cmds
{
#   If using MC/ServiceGuard A.10.10 without any patches, you must uncomment
#   the following line; otherwise, this script will be killed before the startup
#   can be completed.

#   trap "" SIGTERM
#
    export DB2INSTANCE
    if [[ "${DB_TYPE}" = "SINGLE_PARTITION" ]]
    then
      print"\nCleaning up IPC's since we are in SINGLE_PARTITION mode. \n"
      print"\nIf in MULTIPLE_PARTITION mode, tis will cause problems. \n"
      print"\nGo into DB2.sh and correct problem.\n"
      $INSTHOME/sqllib/bin/db2gcf -k -t 30 -i $DB2INSTANCE -p ${DB2NODENUM}
    fi
    $INSTHOME/sqllib/bin/db2gcf -u -t 300 -i $DB2INSTANCE -p ${DB2NODENUM}  -L
    $INSTHOME/sqllib/bin/db2gcf -u -t 300 -i $DB2INSTANCE -p ${DB2NODENUM} -L
    temp_rc=$?

    if [[ ${temp_rc} != 0 ]]
    then
        print "\tERROR code " ${temp_rc} " occurred: Failed to start DB2 server ${DB2INSTANCE}"
    else
	print "\tStarted DB2 instance ${DB2INSTANCE} ${DB2NODENUM} on node  ${HOST}"
        $INSTHOME/sqllib/bin/db2gcf -s -t 300 -i $DB2INSTANCE -p ${DB2NODENUM} -L
        rc=$?
	print "\tdb2gcf -s reports rc of $rc\n"
    fi
}

###############################################################################
# Function: DB2_halt_cmds
#
# The command below is this scripts calling itself with the kill option to
# kill any process that will not die normally after waiting for the TIME_OUT
# period.
###############################################################################
       
function DB2_halt_cmds
{
#   Just in case things are hung, start a process that will wait for the timeout
#   period, then kill any remaining processes.  We'll need to monitor this
#   process (set -m), so we can terminate it later if it is not needed.

    set -m
   ${0} kill ${DB2INSTANCE} ${DB2NODENUM} &

#   If the server is up and running, then we shut it down.
    export DB2INSTANCE
    $INSTHOME/sqllib/bin/db2gcf -d -t 20 -i $DB2INSTANCE -p ${DB2NODENUM}
    temp_rc=$?

    if [[ ${temp_rc} != 0 ]]                                                  
    then
        print "ERROR: Failed to halt DB2 server ${DB2INSTANCE} cleanly: error ${temp_rc}"
    else
	print "Halted DB2 server ${DB2INSTANCE}."
    fi

#   Make sure all processes have gone away before saying shutdown is complete.
#   This stops the other node from starting up the package before it has been
#   stopped and the file system has been unmounted.

    typeset -i c
    typeset -i num_procs=${#MONITOR_PROCESSES[@]}

    while true
    do
	for i in ${MONITOR_PROCESSES[@]}
	do
            ps -fu ${DB2INSTANCE} | grep ${i} | grep "${DB2NODENUM}[ ]*$" | grep -v grep > /dev/null
            if [[ $? != 0 ]]
            then
                print "\n *** ${i} process has stopped. ***\n"
                c=0
                while (( c < $num_procs ))
                do
                    if [[ ${MONITOR_PROCESSES[$c]} = $i ]]
                    then
                        unset MONITOR_PROCESSES[$c]
                        c=$num_procs
                    fi
                    (( c = c + 1 ))
                done
            fi
        done

        if [[ ${MONITOR_PROCESSES[@]} = "" ]]
        then
#           Looks like shutdown was successful, so get rid of the script to
#           kill any hung processes, which we started earlier.  Check to see if
#           the script is still running.  If jobs returns that the script is
#           done, then we don't need to kill it.

            job=$(jobs | grep -v Done)
            if [[ ${job} != "" ]]
            then
                print "Killing the kill_hung_processes script\n"
                kill %1
            fi
            exit 0
        fi

        sleep 5
    done
}

###############################################################################
# Function: terminate
#
# Called when a SIGTERM is trapped.
###############################################################################

function terminate
{
     print "This monitor script has been signaled to terminate\n"
     exit
}

###############################################################################
# Function: kill_hung_processes
#
# In case of a database hang we spawn a background process that will issue
# a SIGKILL to each of the monitored processes.  This allows us to guarantee
# a time limit for clean halt attempts.
###############################################################################

function kill_hung_processes
{
#   Wait for the TIME_OUT period before sending a kill signal.  The TIME_OUT
#   value should be less than the MC/Serviceguard package time out.


    sleep ${TIME_OUT}

    for i in ${MONITOR_PROCESSES[@]}
    do
        id=`ps -fu "${DB2INSTANCE}" | grep "${DB2NODENUM}[ ]*$" | awk '/'${i}'/ { print $2 }'`
        if [[ ${id} != "" ]]
        then
            echo "Killing PID ${id}"
	    kill -9 ${id}
	    if [[ $? != 0 ]]
	    then
	  	print "\n *** ${0} kill_hung_processes function did NOT find process ***\n *** ${i} running. ***\n"
               exit 0
	    else
	  	print "\n *** ${0} kill_hung_processes function did find process ***\n *** ${i} running.  Sent SIGKILL. ***\n"
	    fi
        else
	    print "\n ***  kill_hung_processes function did NOT find ANY processes running. ***\n"
            exit 0
        fi
    done
}
       
###############################################################################
# Function: monitor_processes
##############################################################################

function monitor_processes
{

 while true
 do
    $INSTHOME/sqllib/bin/db2gcf -s -i $DB2INSTANCE -p $DB2NODENUM 2> /dev/null > /dev/null
    rc=$?
    if [ $rc != 0 ]; then
        print "\n\n ERROR:Output of db2gcf:\n"
        $INSTHOME/sqllib/bin/db2gcf -s -i $DB2INSTANCE -p $DB2NODENUM 
        print "\n\n"
        ps -ef
        print "\n *** DB2 has failed.  Aborting DB2. ***"
        set -m
        print "calling ${0} fault ${DB2INSTANCE?} ${DB2NODENUM?}"
        nohup ${0} fault ${DB2INSTANCE?} ${DB2NODENUM?} &    # The script calls itself with the fault option.
        set +m
           sleep 999999
    fi
    sleep ${MONITOR_INTERVAL}
 done

}

###############################################################################
# Function: halt_package
#
# Because there is no move command in MC/Serviceguard so the package has to be
# halted first, then disabled from running on the host, then enabled
# to run in the cluster.
###############################################################################

function halt_package
{
    cmhaltpkg ${PACKAGE_NAME}
    cmmodpkg -d -n ${HOST} ${PACKAGE_NAME}
    sleep 1
    cmmodpkg -e ${PACKAGE_NAME}
}

###############################################################################
# FUNCTION STARTUP SECTION.
#
# Test to see if we are being called to run the application, or halt the
# application.
###############################################################################


trap terminate SIGTERM

if [[ -z ${ACTION} || -z ${DB2INSTANCE} ]] then
	print "\n\n *** USAGE: ${0} ACTION DB2INSTANCE [DB2NODENUM] ***\n\n"
	exit
fi

print "\n *** ${HOST}: ${0} ${ACTION} ${DB2INSTANCE} ${DB2NODENUM} at ${DATE} ***\n"

case ${ACTION} in

    fault)
        halt_package
    ;;

    kill)
        kill_hung_processes
    ;;

    monitor)
        monitor_processes
    ;;

    start)
        DB2_run_cmds
    ;;

    halt)
        DB2_halt_cmds
    ;;
esac
